@@ -26,7 +26,8 @@
 #include <libsoup/soup-server.h>
 #include <libsoup/soup-message.h>
 
-#include "gibber-xmpp-stanza.h"
+#include <wocky/wocky-stanza.h>
+#include <wocky/wocky-meta-porter.h>
 #include "gibber-oob-file-transfer.h"
 #include "gibber-fd-transport.h"
 #include "gibber-namespaces.h"
@@ -86,7 +87,7 @@ static void gibber_oob_file_transfer_receive (GibberFileTransfer *ft,
 static void gibber_oob_file_transfer_cancel (GibberFileTransfer *ft,
     guint error_code);
 static void gibber_oob_file_transfer_received_stanza (GibberFileTransfer *ft,
-    GibberXmppStanza *stanza);
+    WockyStanza *stanza);
 
 static void
 gibber_oob_file_transfer_class_init (
@@ -135,26 +136,27 @@ gibber_oob_file_transfer_finalize (GObject *object)
 }
 
 gboolean
-gibber_oob_file_transfer_is_file_offer (GibberXmppStanza *stanza)
+gibber_oob_file_transfer_is_file_offer (WockyStanza *stanza)
 {
-  GibberStanzaType type;
-  GibberStanzaSubType sub_type;
-  GibberXmppNode *query;
-  GibberXmppNode *url;
+  WockyStanzaType type;
+  WockyStanzaSubType sub_type;
+  WockyNode *query;
+  WockyNode *url;
   const gchar *url_content;
 
-  gibber_xmpp_stanza_get_type_info (stanza, &type, &sub_type);
-  if (type != GIBBER_STANZA_TYPE_IQ ||
-      sub_type != GIBBER_STANZA_SUB_TYPE_SET)
+  wocky_stanza_get_type_info (stanza, &type, &sub_type);
+  if (type != WOCKY_STANZA_TYPE_IQ ||
+      sub_type != WOCKY_STANZA_SUB_TYPE_SET)
     {
       return FALSE;
     }
 
-  query = gibber_xmpp_node_get_child (stanza->node, "query");
+  query = wocky_node_get_child (wocky_stanza_get_top_node (stanza),
+      "query");
   if (query == NULL)
     return FALSE;
 
-  url = gibber_xmpp_node_get_child (query, "url");
+  url = wocky_node_get_child (query, "url");
   if (url == NULL)
     return FALSE;
 
@@ -175,14 +177,17 @@ gibber_oob_file_transfer_is_file_offer (GibberXmppStanza *stanza)
 
 GibberFileTransfer *
 gibber_oob_file_transfer_new_from_stanza_with_from (
-    GibberXmppStanza *stanza,
-    GibberXmppConnection *connection,
-    const gchar *peer_id)
+    WockyStanza *stanza,
+    WockyPorter *porter,
+    WockyContact *contact,
+    const gchar *peer_id,
+    GError **error)
 {
+  WockyNode *node = wocky_stanza_get_top_node (stanza);
   GibberOobFileTransfer *self;
-  GibberXmppNode *query;
-  GibberXmppNode *url_node;
-  GibberXmppNode *desc_node;
+  WockyNode *query;
+  WockyNode *url_node;
+  WockyNode *desc_node;
   const gchar *self_id;
   const gchar *type;
   const gchar *id;
@@ -193,33 +198,83 @@ gibber_oob_file_transfer_new_from_stanza_with_from (
   gchar *url;
   gchar *filename;
 
-  if (strcmp (stanza->node->name, "iq") != 0)
-    return NULL;
+  g_return_val_if_fail (WOCKY_IS_PORTER (porter), NULL);
 
-  self_id = gibber_xmpp_node_get_attribute (stanza->node, "to");
-  if (peer_id == NULL || self_id == NULL)
-    return NULL;
+  if (strcmp (node->name, "iq") != 0)
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "Not an IQ: %s", node->name);
+      return NULL;
+    }
+
+  self_id = wocky_node_get_attribute (node, "to");
+
+  if (peer_id == NULL)
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "No 'from' attribute");
+      return NULL;
+    }
+
+  if (self_id == NULL)
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "No 'to' attribute");
+      return NULL;
+    }
+
+  type = wocky_node_get_attribute (node, "type");
 
-  type = gibber_xmpp_node_get_attribute (stanza->node, "type");
   if (type == NULL || strcmp (type, "set") != 0)
-    return NULL;
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "type != 'set': '%s'", (type == NULL ? "(null)" : type));
+      return NULL;
+    }
+
+  id = wocky_node_get_attribute (node, "id");
 
-  id = gibber_xmpp_node_get_attribute (stanza->node, "id");
   if (id == NULL)
-    return NULL;
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "no 'id' attribute");
+      return NULL;
+    }
+
+  query = wocky_node_get_child (node, "query");
 
-  query = gibber_xmpp_node_get_child (stanza->node, "query");
   if (query == NULL)
-    return NULL;
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "no <query> node");
+      return NULL;
+    }
 
-  url_node = gibber_xmpp_node_get_child (query, "url");
-  if (url_node == NULL || url_node->content == NULL)
-    return NULL;
+  url_node = wocky_node_get_child (query, "url");
+
+  if (url_node == NULL)
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "no <query><url> node");
+      return NULL;
+    }
+
+  if (url_node->content == NULL)
+    {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "<query><url> node has no content");
+      return NULL;
+    }
+
+  ft_type = wocky_node_get_attribute (url_node, "type");
 
-  ft_type = gibber_xmpp_node_get_attribute (url_node, "type");
   if (ft_type != NULL && gibber_strdiff (ft_type, "file"))
-    /* We don't support directory transfer */
-    return NULL;
+    {
+      /* We don't support directory transfer */
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "<url> has a 'type' attribute other than 'file': '%s'", ft_type);
+      return NULL;
+    }
 
   /* The file name is extracted from the address */
   url = g_strdup (url_node->content);
@@ -227,19 +282,21 @@ gibber_oob_file_transfer_new_from_stanza_with_from (
   filename = g_strrstr (url, "/");
   if (filename == NULL)
     {
+      g_set_error (error, WOCKY_XMPP_ERROR, WOCKY_XMPP_ERROR_BAD_REQUEST,
+          "<url> has no '/': '%s'", url);
       g_free (url);
       return NULL;
     }
   filename++; /* move after the last "/" */
   filename = g_uri_unescape_string (filename, NULL);
 
-  desc_node = gibber_xmpp_node_get_child (query, "desc");
+  desc_node = wocky_node_get_child (query, "desc");
   if (desc_node != NULL)
     {
       description = desc_node->content;
     }
 
-  content_type = gibber_xmpp_node_get_attribute (url_node, "mimeType");
+  content_type = wocky_node_get_attribute (url_node, "mimeType");
   if (content_type == NULL)
     {
       content_type = "application/octet-stream";
@@ -250,13 +307,14 @@ gibber_oob_file_transfer_new_from_stanza_with_from (
       "self-id", self_id,
       "peer-id", peer_id,
       "filename", filename,
-      "connection", connection,
+      "porter", porter,
+      "contact", contact,
       "direction", GIBBER_FILE_TRANSFER_DIRECTION_INCOMING,
       "description", description,
       "content-type", content_type,
       NULL);
 
-  size = gibber_xmpp_node_get_attribute (url_node, "size");
+  size = wocky_node_get_attribute (url_node, "size");
   if (size != NULL)
     gibber_file_transfer_set_size (GIBBER_FILE_TRANSFER (self),
       g_ascii_strtoull (size, NULL, 0));
@@ -316,7 +374,7 @@ http_client_finished_chunks_cb (SoupSession *session,
                                 gpointer user_data)
 {
   GibberOobFileTransfer *self = user_data;
-  GibberXmppStanza *stanza;
+  WockyStanza *stanza;
   GError *error = NULL;
   guint64 size;
 
@@ -360,14 +418,12 @@ http_client_finished_chunks_cb (SoupSession *session,
       return;
     }
 
-  stanza = gibber_xmpp_stanza_new ("iq");
-  gibber_xmpp_node_set_attribute (stanza->node, "type", "result");
-  gibber_xmpp_node_set_attribute (stanza->node, "from",
-      GIBBER_FILE_TRANSFER (self)->self_id);
-  gibber_xmpp_node_set_attribute (stanza->node, "to",
-      GIBBER_FILE_TRANSFER (self)->peer_id);
-  gibber_xmpp_node_set_attribute (stanza->node, "id",
-      GIBBER_FILE_TRANSFER (self)->id);
+  stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+      WOCKY_STANZA_SUB_TYPE_RESULT,
+      GIBBER_FILE_TRANSFER (self)->self_id,
+      GIBBER_FILE_TRANSFER (self)->peer_id,
+      WOCKY_NODE_ATTRIBUTE, "id", GIBBER_FILE_TRANSFER (self)->id,
+      NULL);
 
   if (!gibber_file_transfer_send_stanza (GIBBER_FILE_TRANSFER (self), stanza,
         &error))
@@ -415,22 +471,25 @@ gibber_oob_file_transfer_receive (GibberFileTransfer *ft,
       http_client_finished_chunks_cb, self);
 }
 
-static GibberXmppStanza *
+static WockyStanza *
 create_transfer_offer (GibberOobFileTransfer *self,
                        GError **error)
 {
-  GibberXmppConnection *connection;
+  WockyMetaPorter *porter;
+  WockyContact *contact;
+  GSocketConnection *conn;
+  GSocketAddress *address;
+  GInetAddress *addr;
+  GSocketFamily family;
 
   /* local host name */
-  gchar host_name[NI_MAXHOST];
-  struct sockaddr_storage name_addr;
-  socklen_t name_addr_len = sizeof (name_addr);
+  gchar *host_name;
   gchar *host_escaped;
 
-  GibberXmppStanza *stanza;
-  GibberXmppNode *query_node;
-  GibberXmppNode *url_node;
-  GibberXmppNode *desc_node;
+  WockyStanza *stanza;
+  WockyNode *node;
+  WockyNode *query_node;
+  WockyNode *url_node;
 
   gchar *filename_escaped;
   gchar *url;
@@ -438,21 +497,33 @@ create_transfer_offer (GibberOobFileTransfer *self,
 
   guint64 size;
 
-  g_object_get (GIBBER_FILE_TRANSFER (self), "connection", &connection, NULL);
-  if (connection->transport == NULL)
+  g_object_get (GIBBER_FILE_TRANSFER (self),
+      "porter", &porter,
+      "contact", &contact,
+      NULL);
+
+  conn = wocky_meta_porter_borrow_connection (porter, WOCKY_LL_CONTACT (contact));
+
+  g_object_unref (porter);
+  g_object_unref (contact);
+
+  if (conn == NULL)
     {
       g_set_error (error, GIBBER_FILE_TRANSFER_ERROR,
           GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Null transport");
       return NULL;
     }
 
-  gibber_transport_get_sockaddr (connection->transport, &name_addr,
-      &name_addr_len);
-  g_object_unref (connection);
-  getnameinfo ((struct sockaddr *) &name_addr, name_addr_len, host_name,
-      sizeof (host_name), NULL, 0, NI_NUMERICHOST);
+  address = g_socket_connection_get_local_address (conn, NULL);
+  address = gibber_normalize_socket_address (address);
+  addr = g_inet_socket_address_get_address (G_INET_SOCKET_ADDRESS (address));
+  family = g_socket_address_get_family (address);
 
-  if (name_addr.ss_family == AF_INET6)
+  host_name = g_inet_address_to_string (addr);
+
+  g_object_unref (address);
+
+  if (family == G_SOCKET_FAMILY_IPV6)
     {
       /* put brackets around the IP6 */
       host_escaped = g_strdup_printf ("[%s]", host_name);
@@ -463,6 +534,8 @@ create_transfer_offer (GibberOobFileTransfer *self,
       host_escaped = g_strdup (host_name);
     }
 
+  g_free (host_name);
+
   filename_escaped = g_uri_escape_string (GIBBER_FILE_TRANSFER (self)->filename,
       NULL, FALSE);
   url = g_strdup_printf ("http://%s:%d/%s/%s", host_escaped,
@@ -473,24 +546,23 @@ create_transfer_offer (GibberOobFileTransfer *self,
   served_name = g_strdup_printf ("/%s/%s", GIBBER_FILE_TRANSFER (self)->id,
       GIBBER_FILE_TRANSFER (self)->filename);
 
-  stanza = gibber_xmpp_stanza_new ("iq");
-  gibber_xmpp_node_set_attribute (stanza->node, "type", "set");
-  gibber_xmpp_node_set_attribute (stanza->node, "id",
-      GIBBER_FILE_TRANSFER (self)->id);
-  gibber_xmpp_node_set_attribute (stanza->node, "from",
-      GIBBER_FILE_TRANSFER (self)->self_id);
-  gibber_xmpp_node_set_attribute (stanza->node, "to",
-      GIBBER_FILE_TRANSFER (self)->peer_id);
+  stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+      WOCKY_STANZA_SUB_TYPE_SET,
+      GIBBER_FILE_TRANSFER (self)->self_id,
+      GIBBER_FILE_TRANSFER (self)->peer_id,
+      WOCKY_NODE_ATTRIBUTE, "id", GIBBER_FILE_TRANSFER (self)->id,
+      NULL);
+  node = wocky_stanza_get_top_node (stanza);
 
-  query_node = gibber_xmpp_node_add_child_ns (stanza->node, "query",
+  query_node = wocky_node_add_child_ns (node, "query",
       GIBBER_XMPP_NS_IQ_OOB);
 
-  url_node = gibber_xmpp_node_add_child_with_content (query_node, "url", url);
-  gibber_xmpp_node_set_attribute (url_node, "type", "file");
-  gibber_xmpp_node_set_attribute (url_node, "mimeType",
+  url_node = wocky_node_add_child_with_content (query_node, "url", url);
+  wocky_node_set_attribute (url_node, "type", "file");
+  wocky_node_set_attribute (url_node, "mimeType",
       GIBBER_FILE_TRANSFER (self)->content_type);
 
-  desc_node = gibber_xmpp_node_add_child_with_content (query_node, "desc",
+  wocky_node_add_child_with_content (query_node, "desc",
       GIBBER_FILE_TRANSFER (self)->description);
 
   size = gibber_file_transfer_get_size (GIBBER_FILE_TRANSFER (self));
@@ -500,7 +572,7 @@ create_transfer_offer (GibberOobFileTransfer *self,
     {
       gchar *size_str = g_strdup_printf ("%" G_GUINT64_FORMAT,
           size);
-      gibber_xmpp_node_set_attribute (url_node, "size", size_str);
+      wocky_node_set_attribute (url_node, "size", size_str);
       g_free (size_str);
     }
 
@@ -667,58 +739,10 @@ http_server_cb (SoupServer *server,
 }
 
 static void
-gibber_oob_file_transfer_offer (GibberFileTransfer *ft)
+create_and_send_transfer_offer (GibberOobFileTransfer *self)
 {
-  GibberOobFileTransfer *self = GIBBER_OOB_FILE_TRANSFER (ft);
-  GibberXmppStanza *stanza;
   GError *error = NULL;
-
-  /* start the server if not running */
-  /* FIXME we should have only a single server */
-  if (self->priv->server == NULL)
-    {
-      /* FIXME: libsoup can't listen on IPv4 and IPv6 interfaces at the same
-       * time. http://bugzilla.gnome.org/show_bug.cgi?id=522519
-       * We have to check which IP will be send when creating the stanza. */
-        GibberXmppConnection *connection;
-        struct sockaddr_storage name_addr;
-        socklen_t name_addr_len = sizeof (name_addr);
-
-        g_object_get (GIBBER_FILE_TRANSFER (self), "connection", &connection,
-            NULL);
-        if (connection->transport == NULL)
-          {
-            g_set_error (&error, GIBBER_FILE_TRANSFER_ERROR,
-                GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Null transport");
-            gibber_file_transfer_emit_error (GIBBER_FILE_TRANSFER (self),
-                error);
-            g_error_free (error);
-            return;
-          }
-
-        gibber_transport_get_peeraddr (connection->transport, &name_addr,
-            &name_addr_len);
-        g_object_unref (connection);
-
-        if (name_addr.ss_family == AF_INET6)
-          {
-            /* IPv6 server */
-            SoupAddress *addr;
-
-            addr = soup_address_new_any (SOUP_ADDRESS_FAMILY_IPV6, 0);
-            self->priv->server = soup_server_new (SOUP_SERVER_INTERFACE,
-                addr, NULL);
-
-            g_object_unref (addr);
-          }
-        else
-          {
-            /* IPv4 server */
-            self->priv->server = soup_server_new (NULL, NULL);
-          }
-
-      soup_server_run_async (self->priv->server);
-    }
+  WockyStanza *stanza;
 
   stanza = create_transfer_offer (self, &error);
   if (stanza == NULL)
@@ -739,6 +763,115 @@ gibber_oob_file_transfer_offer (GibberFileTransfer *ft)
   g_object_unref (stanza);
 }
 
+static void
+porter_open_cb (GObject *source_object,
+    GAsyncResult *result,
+    gpointer user_data)
+{
+  WockyMetaPorter *porter = WOCKY_META_PORTER (source_object);
+  GibberOobFileTransfer *self = GIBBER_OOB_FILE_TRANSFER (user_data);
+  GError *error = NULL;
+
+  WockyContact *contact;
+
+  GSocketConnection *conn;
+  GSocketAddress *address;
+  GSocketFamily family;
+
+  if (!wocky_meta_porter_open_finish (porter, result, &error))
+    {
+      DEBUG ("Failed to open connection: %s", error->message);
+      g_clear_error (&error);
+
+      g_set_error (&error, GIBBER_FILE_TRANSFER_ERROR,
+          GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Couldn't open connection");
+      gibber_file_transfer_emit_error (GIBBER_FILE_TRANSFER (self),
+          error);
+      g_error_free (error);
+      return;
+    }
+
+  g_object_get (GIBBER_FILE_TRANSFER (self),
+      "contact", &contact,
+      NULL);
+
+  /* FIXME we should have only a single server */
+
+  /* FIXME: libsoup can't listen on IPv4 and IPv6 interfaces at the same
+   * time. http://bugzilla.gnome.org/show_bug.cgi?id=522519
+   * We have to check which IP will be send when creating the stanza. */
+
+  conn = wocky_meta_porter_borrow_connection (porter,
+      WOCKY_LL_CONTACT (contact));
+
+  if (conn == NULL)
+    {
+      g_set_error (&error, GIBBER_FILE_TRANSFER_ERROR,
+          GIBBER_FILE_TRANSFER_ERROR_NOT_CONNECTED, "Null transport");
+      gibber_file_transfer_emit_error (GIBBER_FILE_TRANSFER (self),
+          error);
+      g_error_free (error);
+      goto out;
+    }
+
+  address = g_socket_connection_get_remote_address (conn, NULL);
+  family = g_socket_address_get_family (address);
+  g_object_unref (address);
+
+  if (family == G_SOCKET_FAMILY_IPV6)
+    {
+      /* IPv6 server */
+      SoupAddress *addr;
+
+      addr = soup_address_new_any (SOUP_ADDRESS_FAMILY_IPV6, 0);
+      self->priv->server = soup_server_new (SOUP_SERVER_INTERFACE,
+          addr, NULL);
+
+      g_object_unref (addr);
+    }
+  else
+    {
+      /* IPv4 server */
+      self->priv->server = soup_server_new (NULL, NULL);
+    }
+
+  soup_server_run_async (self->priv->server);
+
+  create_and_send_transfer_offer (self);
+
+out:
+  /* this was reffed when calling open_async */
+  wocky_meta_porter_unhold (porter, contact);
+  g_object_unref (contact);
+}
+
+static void
+gibber_oob_file_transfer_offer (GibberFileTransfer *ft)
+{
+  GibberOobFileTransfer *self = GIBBER_OOB_FILE_TRANSFER (ft);
+  WockyMetaPorter *porter;
+  WockyContact *contact;
+
+  if (self->priv->server != NULL)
+    {
+      create_and_send_transfer_offer (self);
+      return;
+    }
+
+  /* we need to create the soup server */
+
+  g_object_get (ft,
+      "porter", &porter,
+      "contact", &contact,
+      NULL);
+
+  wocky_meta_porter_open_async (porter, WOCKY_LL_CONTACT (contact),
+      NULL, porter_open_cb, ft);
+
+  g_object_unref (contact);
+  g_object_unref (porter);
+}
+
 static void
 http_server_wrote_chunk_cb (SoupMessage *msg,
                             gpointer user_data)
@@ -776,10 +909,10 @@ gibber_oob_file_transfer_cancel (GibberFileTransfer *ft,
                                  guint error_code)
 {
   GibberOobFileTransfer *self = GIBBER_OOB_FILE_TRANSFER (ft);
-  GibberXmppStanza *stanza;
-  GibberXmppNode *query;
-  GibberXmppNode *error_node;
-  GibberXmppNode *error_desc;
+  WockyStanza *stanza;
+  WockyNode *node;
+  WockyNode *query;
+  WockyNode *error_node;
   gchar *code_string;
 
   if (self->priv->cancelled)
@@ -791,31 +924,33 @@ gibber_oob_file_transfer_cancel (GibberFileTransfer *ft,
      * sender cancelled the transfer. */
     return;
 
-  stanza = gibber_xmpp_stanza_new ("iq");
-  gibber_xmpp_node_set_attribute (stanza->node, "type", "error");
-  gibber_xmpp_node_set_attribute (stanza->node, "from", ft->self_id);
-  gibber_xmpp_node_set_attribute (stanza->node, "to", ft->peer_id);
-  gibber_xmpp_node_set_attribute (stanza->node, "id", ft->id);
+  stanza = wocky_stanza_build (WOCKY_STANZA_TYPE_IQ,
+      WOCKY_STANZA_SUB_TYPE_ERROR,
+      GIBBER_FILE_TRANSFER (self)->self_id,
+      GIBBER_FILE_TRANSFER (self)->peer_id,
+      WOCKY_NODE_ATTRIBUTE, "id", GIBBER_FILE_TRANSFER (self)->id,
+      NULL);
+  node = wocky_stanza_get_top_node (stanza);
 
-  query = gibber_xmpp_node_add_child_ns (stanza->node, "query",
+  query = wocky_node_add_child_ns (node, "query",
       GIBBER_XMPP_NS_IQ_OOB);
-  gibber_xmpp_node_add_child_with_content (query, "url", self->priv->url);
+  wocky_node_add_child_with_content (query, "url", self->priv->url);
 
-  error_node = gibber_xmpp_node_add_child (stanza->node, "error");
+  error_node = wocky_node_add_child (node, "error");
   code_string = g_strdup_printf ("%d", error_code);
 
   switch (error_code)
     {
       case HTTP_STATUS_CODE_NOT_FOUND:
-        gibber_xmpp_node_set_attribute (error_node, "code", code_string);
-        gibber_xmpp_node_set_attribute (error_node, "type", "cancel");
-        error_desc = gibber_xmpp_node_add_child_ns (error_node,
+        wocky_node_set_attribute (error_node, "code", code_string);
+        wocky_node_set_attribute (error_node, "type", "cancel");
+        wocky_node_add_child_ns (error_node,
             "item-not-found", GIBBER_XMPP_NS_STANZAS);
         break;
       case HTTP_STATUS_CODE_NOT_ACCEPTABLE:
-        gibber_xmpp_node_set_attribute (error_node, "code", code_string);
-        gibber_xmpp_node_set_attribute (error_node, "type", "modify");
-        error_desc = gibber_xmpp_node_add_child_ns (error_node,
+        wocky_node_set_attribute (error_node, "code", code_string);
+        wocky_node_set_attribute (error_node, "type", "modify");
+        wocky_node_add_child_ns (error_node,
             "not-acceptable", GIBBER_XMPP_NS_STANZAS);
         break;
       default:
@@ -831,16 +966,17 @@ gibber_oob_file_transfer_cancel (GibberFileTransfer *ft,
 
 static void
 gibber_oob_file_transfer_received_stanza (GibberFileTransfer *ft,
-                                          GibberXmppStanza *stanza)
+                                          WockyStanza *stanza)
 {
   GibberOobFileTransfer *self = GIBBER_OOB_FILE_TRANSFER (ft);
+  WockyNode *node = wocky_stanza_get_top_node (stanza);
   const gchar *type;
-  GibberXmppNode *error_node;
+  WockyNode *error_node;
 
-  if (strcmp (stanza->node->name, "iq") != 0)
+  if (strcmp (node->name, "iq") != 0)
     return;
 
-  type = gibber_xmpp_node_get_attribute (stanza->node, "type");
+  type = wocky_node_get_attribute (node, "type");
   if (type == NULL)
     return;
 
@@ -850,7 +986,7 @@ gibber_oob_file_transfer_received_stanza (GibberFileTransfer *ft,
       return;
     }
 
-  error_node = gibber_xmpp_node_get_child (stanza->node, "error");
+  error_node = wocky_node_get_child (node, "error");
   if (error_node != NULL)
     {
       GError *error = NULL;
@@ -859,10 +995,10 @@ gibber_oob_file_transfer_received_stanza (GibberFileTransfer *ft,
       self->priv->cancelled = TRUE;
 
       /* FIXME copy the error handling code from gabble */
-      error_code_str = gibber_xmpp_node_get_attribute (error_node, "code");
+      error_code_str = wocky_node_get_attribute (error_node, "code");
       if (error_code_str == NULL)
         /* iChat uses the 'type' attribute to transmit the error code */
-        error_code_str = gibber_xmpp_node_get_attribute (error_node, "type");
+        error_code_str = wocky_node_get_attribute (error_node, "type");
 
       if (error_code_str != NULL && g_ascii_strtoll (error_code_str, NULL,
             10) == HTTP_STATUS_CODE_NOT_ACCEPTABLE)
