@@ -25,7 +25,7 @@
 
 #include "gibber-r-multicast-sender.h"
 #include "gibber-util.h"
-#include "signals-marshal.h"
+#include "gibber-signals-marshal.h"
 
 #define DEBUG_FLAG DEBUG_RMULTICAST_SENDER
 #include "gibber-debug.h"
@@ -155,7 +155,7 @@ void
 gibber_r_multicast_sender_group_free (GibberRMulticastSenderGroup *group)
 {
   GHashTable *h;
-  int i;
+  guint i;
 
   g_assert (group->popping == FALSE);
 
@@ -210,7 +210,7 @@ static gboolean
 find_by_name (gpointer key, gpointer value, gpointer user_data)
 {
   GibberRMulticastSender *sender = GIBBER_R_MULTICAST_SENDER (value);
-  const gchar *name = (gchar *)user_data;
+  const gchar *name = (gchar *) user_data;
 
   if (sender->state == GIBBER_R_MULTICAST_SENDER_STATE_PENDING_REMOVAL)
     return FALSE;
@@ -239,7 +239,7 @@ static void
 gibber_r_multicast_sender_group_gc_acks (GibberRMulticastSenderGroup *group,
     guint32 sender_id)
 {
-  int i;
+  guint i;
   GibberRMulticastSender *s;
   /* If there are no more senders for sender_id in the list, clean up the acks
    */
@@ -297,7 +297,7 @@ static void
 create_sender_array (gpointer key, gpointer value, gpointer user_data)
 {
   GibberRMulticastSender *sender = GIBBER_R_MULTICAST_SENDER (value);
-  GArray *array = (GArray *)user_data;
+  GArray *array = (GArray *) user_data;
   AckInfo info;
 
   if (sender->state >= GIBBER_R_MULTICAST_SENDER_STATE_STOPPED)
@@ -313,8 +313,8 @@ static void
 update_sender_acks (gpointer key, gpointer value, gpointer user_data)
 {
   GibberRMulticastSender *sender = GIBBER_R_MULTICAST_SENDER (value);
-  GArray *array = (GArray *)user_data;
-  gint i;
+  GArray *array = (GArray *) user_data;
+  guint i;
 
   if (sender->state >= GIBBER_R_MULTICAST_SENDER_STATE_STOPPED)
     return;
@@ -428,7 +428,7 @@ can_gc_sender (GibberRMulticastSenderGroup *group,
   return ret == NULL;
 }
 
-void
+static void
 gibber_r_multicast_sender_group_gc (GibberRMulticastSenderGroup *group)
 {
   GArray *array;
@@ -458,7 +458,7 @@ gibber_r_multicast_sender_group_gc (GibberRMulticastSenderGroup *group)
 
       s = GIBBER_R_MULTICAST_SENDER (g_ptr_array_index (group->pending_removal,
           i));
-      if (can_gc_sender (group, s));
+      if (can_gc_sender (group, s))
         {
           g_ptr_array_remove_index_fast (group->pending_removal, i);
           gibber_r_multicast_sender_group_gc_acks (group, s->id);
@@ -475,7 +475,7 @@ gibber_r_multicast_sender_group_push_packet (
 {
   gboolean handled = FALSE;
   GibberRMulticastSender *sender;
-  int i;
+  guint i;
 
   if (packet->type == PACKET_TYPE_WHOIS_REQUEST)
     sender = gibber_r_multicast_sender_group_lookup (group,
@@ -632,7 +632,7 @@ typedef struct {
 static void
 packet_info_free (gpointer data)
 {
-  PacketInfo *p = (PacketInfo *)data;
+  PacketInfo *p = (PacketInfo *) data;
   if (p->packet != NULL) {
     g_object_unref (p->packet);
   }
@@ -695,10 +695,10 @@ gibber_r_multicast_sender_class_init (
 {
   GObjectClass *object_class =
       G_OBJECT_CLASS (gibber_r_multicast_sender_class);
+  GParamSpec *param_spec;
 
   g_type_class_add_private (gibber_r_multicast_sender_class,
       sizeof (GibberRMulticastSenderPrivate));
-  GParamSpec *param_spec;
 
   object_class->dispose = gibber_r_multicast_sender_dispose;
   object_class->finalize = gibber_r_multicast_sender_finalize;
@@ -773,9 +773,7 @@ gibber_r_multicast_sender_class_init (
       "Sender Group",
       "Group of senders",
       G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_WRITABLE       |
-      G_PARAM_STATIC_NAME    |
-      G_PARAM_STATIC_BLURB);
+      G_PARAM_WRITABLE       | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_property (object_class, PROP_SENDER_GROUP,
       param_spec);
@@ -1034,7 +1032,7 @@ name_discovered (GibberRMulticastSender *self, const gchar *name)
 static gboolean
 request_repair (gpointer data)
 {
-  PacketInfo *info = (PacketInfo *)data;
+  PacketInfo *info = (PacketInfo *) data;
 
   DEBUG_SENDER (info->sender, "Sending out repair request for 0x%x",
     info->packet_id);
@@ -1084,7 +1082,7 @@ schedule_repair (GibberRMulticastSender *sender, guint32 id)
 static gboolean
 do_repair (gpointer data)
 {
-  PacketInfo *info = (PacketInfo *)data;
+  PacketInfo *info = (PacketInfo *) data;
 
   g_assert (info != NULL && info->packet != NULL);
 
@@ -1668,7 +1666,7 @@ pop_packets (GibberRMulticastSender *sender)
   g_object_unref (sender);
 }
 
-void
+static void
 insert_packet (GibberRMulticastSender *sender, GibberRMulticastPacket *packet)
 {
   GibberRMulticastSenderPrivate *priv =
@@ -1771,7 +1769,7 @@ gibber_r_multicast_sender_update_end (GibberRMulticastSender *sender,
 
   g_assert (sender->state == GIBBER_R_MULTICAST_SENDER_STATE_FAILED);
 
-  if  (gibber_r_multicast_packet_diff (priv->end_point, packet_id) >= 0);
+  if  (gibber_r_multicast_packet_diff (priv->end_point, packet_id) >= 0)
     {
       DEBUG_SENDER (sender, "Updating end to %x", packet_id);
       priv->end_point = packet_id;
@@ -2061,7 +2059,7 @@ gibber_r_multicast_sender_get_ackinfo (GibberRMulticastSender *sender,
   GibberRMulticastSenderPrivate *priv =
     GIBBER_R_MULTICAST_SENDER_GET_PRIVATE (sender);
 
-  return (AckInfo *)g_hash_table_lookup (priv->acks, &sender_id);
+  return (AckInfo *) g_hash_table_lookup (priv->acks, &sender_id);
 }
 
 void
@@ -2122,7 +2120,7 @@ gibber_r_multicast_sender_release_data (GibberRMulticastSender *sender)
 static void
 stop_packet (gpointer key, gpointer value, gpointer user_data)
 {
-  PacketInfo *p = (PacketInfo *)value;
+  PacketInfo *p = (PacketInfo *) value;
 
   if (p->timeout != 0)
     {
