@@ -6,7 +6,7 @@
 
 #include <glib.h>
 
-#include <gibber/gibber-xmpp-stanza.h>
+#include <wocky/wocky-stanza.h>
 
 #ifdef ENABLE_DEBUG
 
@@ -33,6 +33,7 @@ typedef enum
   DEBUG_DISCOVERY      = 1 << 18,
   DEBUG_OLPC_ACTIVITY  = 1 << 19,
   DEBUG_FT             = 1 << 20,
+  DEBUG_PLUGIN         = 1 << 21,
 } DebugFlags;
 
 void debug_set_flags_from_env (void);
