@@ -5,9 +5,11 @@
 #include <telepathy-glib/run.h>
 #include <telepathy-glib/debug.h>
 
-#include "salut-connection-manager.h"
-#include "salut-avahi-discovery-client.h"
+#include "connection-manager.h"
+#include "avahi-discovery-client.h"
 #include "debug.h"
+#include "plugin-loader.h"
+#include "symbol-hacks.h"
 
 static TpBaseConnectionManager *
 salut_create_connection_manager (void)
@@ -22,10 +24,14 @@ int
 main (int argc, char **argv)
 {
   GLogLevelFlags fatal_mask;
+  gint ret;
+  SalutPluginLoader *loader;
 
   g_type_init ();
   g_thread_init (NULL);
 
+  salut_symbol_hacks ();
+
   /* treat criticals as, well, critical */
   fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
   fatal_mask |= G_LOG_LEVEL_CRITICAL;
@@ -39,9 +45,15 @@ main (int argc, char **argv)
     tp_debug_set_persistent (TRUE);
 #endif
 
-  return tp_run_connection_manager ("telepathy-salut", VERSION,
+  loader = salut_plugin_loader_dup ();
+
+  ret = tp_run_connection_manager ("telepathy-salut", VERSION,
                                     salut_create_connection_manager,
                                     argc, argv);
+
+  g_object_unref (loader);
+
+  return ret;
 }
 
 
