@@ -44,7 +44,7 @@
 
 #define DEBUG_FLAG DEBUG_TUBES
 #include "debug.h"
-#include "salut-connection.h"
+#include "connection.h"
 #include "tube-iface.h"
 #include "sha1/sha1-util.h"
 
@@ -432,7 +432,7 @@ create_dbus_server (SalutTubeDBus *self,
       g_free (priv->socket_path);
 
       generate_ascii_string (8, suffix);
-      priv->socket_path = g_strdup_printf ("%s/dbus-salut-%.8s",
+      priv->socket_path = g_strdup_printf ("%s/dbus-%.8s",
           g_get_tmp_dir (), suffix);
       priv->dbus_srv_addr = g_strdup_printf ("unix:path=%s",
           priv->socket_path);
@@ -508,7 +508,7 @@ unref_handle_foreach (gpointer key,
   tp_handle_unref (contact_repo, handle);
 }
 
-static TpTubeState
+static TpTubeChannelState
 get_tube_state (SalutTubeDBus *self)
 {
   SalutTubeDBusPrivate *priv = SALUT_TUBE_DBUS_GET_PRIVATE (self);
@@ -904,7 +904,8 @@ salut_tube_dbus_constructor (GType type,
   GObject *obj;
   SalutTubeDBus *self;
   SalutTubeDBusPrivate *priv;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
+  TpBaseConnection *base_conn;
   TpHandleRepoIface *contact_repo;
   TpHandleRepoIface *handles_repo;
   TpSocketAccessControl access_control;
@@ -915,18 +916,18 @@ salut_tube_dbus_constructor (GType type,
 
   priv = SALUT_TUBE_DBUS_GET_PRIVATE (self);
 
-  handles_repo = tp_base_connection_get_handles
-      ((TpBaseConnection *) priv->conn, priv->handle_type);
+  base_conn = TP_BASE_CONNECTION (priv->conn);
+  handles_repo = tp_base_connection_get_handles (base_conn, priv->handle_type);
 
   /* Ref the initiator handle */
   g_assert (priv->conn != NULL);
   g_assert (priv->initiator != 0);
-  contact_repo = tp_base_connection_get_handles
-      ((TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  contact_repo = tp_base_connection_get_handles (base_conn,
+      TP_HANDLE_TYPE_CONTACT);
   tp_handle_ref (contact_repo, priv->initiator);
 
-  bus = tp_get_bus ();
-  dbus_g_connection_register_g_object (bus, priv->object_path, obj);
+  bus = tp_base_connection_get_dbus_daemon (base_conn);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 
   DEBUG ("Registering at '%s'", priv->object_path);
 
