@@ -42,30 +42,28 @@
 #include <telepathy-glib/svc-channel.h>
 #include <telepathy-glib/svc-generic.h>
 
-#include <gibber/gibber-xmpp-stanza.h>
+#include <wocky/wocky-stanza.h>
 #include <gibber/gibber-namespaces.h>
 #include <gibber/gibber-bytestream-direct.h>
 #include <gibber/gibber-bytestream-iface.h>
 #include <gibber/gibber-bytestream-oob.h>
 #include <gibber/gibber-fd-transport.h>
-#include <gibber/gibber-iq-helper.h>
 #include <gibber/gibber-listener.h>
 #include <gibber/gibber-namespaces.h>
 #include <gibber/gibber-tcp-transport.h>
 #include <gibber/gibber-transport.h>
 #include <gibber/gibber-unix-transport.h>
-#include <gibber/gibber-xmpp-stanza.h>
+#include <wocky/wocky-stanza.h>
 
 #define DEBUG_FLAG DEBUG_TUBES
 
 #include "debug.h"
-#include "salut-signals-marshal.h"
-#include "salut-connection.h"
+#include "signals-marshal.h"
+#include "connection.h"
 #include "tube-iface.h"
-#include "salut-si-bytestream-manager.h"
-#include "salut-contact-manager.h"
-#include "salut-tubes-channel.h"
-#include "salut-xmpp-connection-manager.h"
+#include "si-bytestream-manager.h"
+#include "contact-manager.h"
+#include "tubes-channel.h"
 
 static void tube_iface_init (gpointer g_iface, gpointer iface_data);
 static void channel_iface_init (gpointer g_iface, gpointer iface_data);
@@ -141,7 +139,6 @@ enum
   PROP_ADDRESS,
   PROP_ACCESS_CONTROL,
   PROP_ACCESS_CONTROL_PARAM,
-  PROP_XMPP_CONNECTION_MANAGER,
   PROP_PORT,
   PROP_IQ_REQ,
   PROP_CHANNEL_DESTROYED,
@@ -165,8 +162,7 @@ struct _SalutTubeStreamPrivate
   TpHandle self_handle;
   guint id;
   guint port;
-  GibberXmppConnection *xmpp_connection;
-  GibberXmppStanza *iq_req;
+  WockyStanza *iq_req;
   gchar *object_path;
 
   /* Bytestreams for MUC tubes (using stream initiation) or 1-1 tubes (using
@@ -216,10 +212,6 @@ struct _SalutTubeStreamPrivate
 
   gboolean closed;
 
-  /* we need to send an iq stanza to close the tube on 1-1 tube */
-  GibberIqHelper *iq_helper;
-  SalutXmppConnectionManager *xmpp_connection_manager;
-
   gboolean offer_needed;
 
   gboolean dispose_has_run;
@@ -231,16 +223,6 @@ struct _SalutTubeStreamPrivate
 static void data_received_cb (GibberBytestreamIface *ibb, TpHandle sender,
     GString *data, gpointer user_data);
 
-static void xmpp_connection_manager_new_connection_cb (
-    SalutXmppConnectionManager *mgr, GibberXmppConnection *conn,
-    SalutContact *new_contact, gpointer user_data);
-
-static void xmpp_connection_manager_connection_closed_cb (
-    SalutXmppConnectionManager *mgr, GibberXmppConnection *conn,
-    SalutContact *old_contact, gpointer user_data);
-
-static void ensure_iq_helper (SalutTubeStream *tube);
-
 static void salut_tube_stream_add_bytestream (SalutTubeIface *tube,
     GibberBytestreamIface *bytestream);
 
@@ -531,8 +513,9 @@ start_stream_initiation (SalutTubeStream *self,
                          GError **error)
 {
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
-  GibberXmppNode *node, *si_node;
-  GibberXmppStanza *msg;
+  WockyNode *node, *si_node;
+  WockyStanza *msg;
+  WockyNode *msg_node;
   TpHandleRepoIface *contact_repo;
   const gchar *jid;
   gchar *stream_id, *id_str;
@@ -553,8 +536,9 @@ start_stream_initiation (SalutTubeStream *self,
 
   msg = salut_si_bytestream_manager_make_stream_init_iq (priv->conn->name, jid,
       stream_id, GIBBER_TELEPATHY_NS_TUBES);
+  msg_node = wocky_stanza_get_top_node (msg);
 
-  si_node = gibber_xmpp_node_get_child_ns (msg->node, "si", GIBBER_XMPP_NS_SI);
+  si_node = wocky_node_get_child_ns (msg_node, "si", GIBBER_XMPP_NS_SI);
   g_assert (si_node != NULL);
 
   id_str = g_strdup_printf ("%u", priv->id);
@@ -562,12 +546,12 @@ start_stream_initiation (SalutTubeStream *self,
   g_assert (priv->handle_type == TP_HANDLE_TYPE_ROOM);
 
   /* FIXME: this needs standardizing */
-  node = gibber_xmpp_node_add_child (si_node, "muc-stream");
-  gibber_xmpp_node_set_attribute (node, "muc", tp_handle_inspect (
+  node = wocky_node_add_child_ns (si_node, "muc-stream",
+      GIBBER_TELEPATHY_NS_TUBES);
+  wocky_node_set_attribute (node, "muc", tp_handle_inspect (
         room_repo, priv->handle));
 
-  gibber_xmpp_node_set_ns (node, GIBBER_TELEPATHY_NS_TUBES);
-  gibber_xmpp_node_set_attribute (node, "tube", id_str);
+  wocky_node_set_attribute (node, "tube", id_str);
 
   data = g_slice_new (struct _extra_bytestream_negotiate_cb_data);
   data->self = self;
@@ -591,6 +575,8 @@ start_stream_initiation (SalutTubeStream *self,
     }
   else
     {
+      wocky_stanza_set_to_contact (msg, WOCKY_CONTACT (contact));
+
       result = salut_si_bytestream_manager_negotiate_stream (
         si_bytestream_mgr,
         contact,
@@ -864,7 +850,7 @@ tube_stream_open (SalutTubeStream *self,
       int ret;
 
       generate_ascii_string (8, suffix);
-      path = g_strdup_printf ("/tmp/stream-salut-%.8s", suffix);
+      path = g_strdup_printf ("/tmp/stream-%.8s", suffix);
 
       DEBUG ("create socket: %s", path);
 
@@ -1071,18 +1057,6 @@ salut_tube_stream_dispose (GObject *object)
       priv->contact_listener = NULL;
     }
 
-  if (priv->iq_helper != NULL)
-    {
-      g_object_unref (priv->iq_helper);
-      priv->iq_helper = NULL;
-    }
-
-  if (priv->xmpp_connection_manager != NULL)
-    {
-      g_object_unref (priv->xmpp_connection_manager);
-      priv->xmpp_connection_manager = NULL;
-    }
-
   priv->dispose_has_run = TRUE;
 
   if (G_OBJECT_CLASS (salut_tube_stream_parent_class)->dispose)
@@ -1196,9 +1170,6 @@ salut_tube_stream_get_property (GObject *object,
       case PROP_ACCESS_CONTROL_PARAM:
         g_value_set_pointer (value, priv->access_control_param);
         break;
-      case PROP_XMPP_CONNECTION_MANAGER:
-        g_value_set_object (value, priv->xmpp_connection_manager);
-        break;
       case PROP_PORT:
         g_value_set_uint (value, priv->port);
         break;
@@ -1358,12 +1329,6 @@ salut_tube_stream_set_property (GObject *object,
                 g_value_get_pointer (value));
           }
         break;
-      case PROP_XMPP_CONNECTION_MANAGER:
-        priv->xmpp_connection_manager = g_value_get_object (value);
-        /* xmpp_connection_manager is set only for 1-1 tubes */
-        if (priv->xmpp_connection_manager != NULL)
-          g_object_ref (priv->xmpp_connection_manager);
-        break;
       case PROP_PORT:
         priv->port = g_value_get_uint (value);
         break;
@@ -1378,81 +1343,6 @@ salut_tube_stream_set_property (GObject *object,
     }
 }
 
-static void
-xmpp_connection_manager_connection_closed_cb (SalutXmppConnectionManager *mgr,
-                                              GibberXmppConnection *conn,
-                                              SalutContact *old_contact,
-                                              gpointer user_data)
-{
-  SalutTubeStream *self = SALUT_TUBE_STREAM (user_data);
-  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
-  SalutContactManager *contact_mgr;
-  SalutContact *contact;
-
-  g_object_get (priv->conn, "contact-manager", &contact_mgr, NULL);
-  g_assert (contact_mgr != NULL);
-
-  contact = salut_contact_manager_get_contact (contact_mgr,
-      priv->handle);
-
-  if (contact != old_contact)
-    {
-      /* This connection is not for this tube */
-      g_object_unref (contact);
-      g_object_unref (contact_mgr);
-      return;
-    }
-
-  g_assert (priv->xmpp_connection == conn);
-  priv->xmpp_connection = NULL;
-
-  g_signal_handlers_disconnect_matched (priv->xmpp_connection_manager,
-      G_SIGNAL_MATCH_DATA, 0, 0, NULL, NULL, self);
-
-  g_signal_connect (priv->xmpp_connection_manager, "new-connection",
-      G_CALLBACK (xmpp_connection_manager_new_connection_cb), self);
-
-  g_object_unref (contact);
-  g_object_unref (contact_mgr);
-}
-
-static void
-xmpp_connection_manager_new_connection_cb (SalutXmppConnectionManager *mgr,
-                                           GibberXmppConnection *conn,
-                                           SalutContact *new_contact,
-                                           gpointer user_data)
-{
-  SalutTubeStream *self = SALUT_TUBE_STREAM (user_data);
-  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
-  SalutContactManager *contact_mgr;
-  SalutContact *contact;
-
-  g_object_get (priv->conn, "contact-manager", &contact_mgr, NULL);
-  g_assert (contact_mgr != NULL);
-
-  contact = salut_contact_manager_get_contact (contact_mgr,
-      priv->handle);
-
-  if (contact != new_contact)
-    {
-      /* This new connection is not for this tube */
-      g_object_unref (contact);
-      g_object_unref (contact_mgr);
-      return;
-    }
-
-  DEBUG ("pending connection fully open");
-
-  g_assert (conn != NULL);
-  priv->xmpp_connection = conn;
-
-  g_signal_connect (priv->xmpp_connection_manager, "connection-closed",
-      G_CALLBACK (xmpp_connection_manager_connection_closed_cb), self);
-
-  g_object_unref (contact);
-  g_object_unref (contact_mgr);
-}
-
 static GObject *
 salut_tube_stream_constructor (GType type,
                                 guint n_props,
@@ -1461,7 +1351,8 @@ salut_tube_stream_constructor (GType type,
   GObject *obj;
   SalutTubeStreamPrivate *priv;
   TpHandleRepoIface *contact_repo;
-  DBusGConnection *bus;
+  TpDBusDaemon *bus;
+  TpBaseConnection *base_conn;
 
   obj = G_OBJECT_CLASS (salut_tube_stream_parent_class)->
            constructor (type, n_props, props);
@@ -1469,10 +1360,10 @@ salut_tube_stream_constructor (GType type,
   priv = SALUT_TUBE_STREAM_GET_PRIVATE (SALUT_TUBE_STREAM (obj));
 
   /* Ref the initiator handle */
-  g_assert (priv->conn != NULL);
+  base_conn = TP_BASE_CONNECTION (priv->conn);
   g_assert (priv->initiator != 0);
-  contact_repo = tp_base_connection_get_handles
-      ((TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
+  contact_repo = tp_base_connection_get_handles (base_conn,
+      TP_HANDLE_TYPE_CONTACT);
   tp_handle_ref (contact_repo, priv->initiator);
 
   if (priv->initiator == priv->self_handle)
@@ -1487,20 +1378,8 @@ salut_tube_stream_constructor (GType type,
       priv->state = TP_TUBE_CHANNEL_STATE_LOCAL_PENDING;
     }
 
-  if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
-    {
-      g_assert (priv->xmpp_connection_manager != NULL);
-      g_signal_connect (priv->xmpp_connection_manager, "new-connection",
-          G_CALLBACK (xmpp_connection_manager_new_connection_cb), obj);
-      ensure_iq_helper (SALUT_TUBE_STREAM (obj));
-    }
-  else
-    {
-      g_assert (priv->xmpp_connection_manager == NULL);
-    }
-
-  bus = tp_get_bus ();
-  dbus_g_connection_register_g_object (bus, priv->object_path, obj);
+  bus = tp_base_connection_get_dbus_daemon (base_conn);
+  tp_dbus_daemon_register_object (bus, priv->object_path, obj);
 
   DEBUG ("Registering at '%s'", priv->object_path);
 
@@ -1649,19 +1528,6 @@ salut_tube_stream_class_init (SalutTubeStreamClass *salut_tube_stream_class)
       G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_STATIC_NAME);
   g_object_class_install_property (object_class, PROP_TARGET_ID, param_spec);
 
-  param_spec = g_param_spec_object (
-      "xmpp-connection-manager",
-      "SalutXmppConnectionManager object",
-      "Salut XMPP Connection manager used for this tube channel in case of "
-          "1-1 tube",
-      SALUT_TYPE_XMPP_CONNECTION_MANAGER,
-      G_PARAM_CONSTRUCT_ONLY |
-      G_PARAM_READWRITE |
-      G_PARAM_STATIC_NICK |
-      G_PARAM_STATIC_BLURB);
-  g_object_class_install_property (object_class, PROP_XMPP_CONNECTION_MANAGER,
-      param_spec);
-
   param_spec = g_param_spec_uint (
       "port",
       "port on the initiator's CM",
@@ -1802,7 +1668,6 @@ data_received_cb (GibberBytestreamIface *bytestream,
 SalutTubeStream *
 salut_tube_stream_new (SalutConnection *conn,
                        SalutTubesChannel *tubes_channel,
-                       SalutXmppConnectionManager *xmpp_connection_manager,
                        TpHandle handle,
                        TpHandleType handle_type,
                        TpHandle self_handle,
@@ -1812,7 +1677,7 @@ salut_tube_stream_new (SalutConnection *conn,
                        GHashTable *parameters,
                        guint id,
                        guint portnum,
-                       GibberXmppStanza *iq_req)
+                       WockyStanza *iq_req)
 {
   SalutTubeStream *obj;
   char *object_path;
@@ -1824,7 +1689,6 @@ salut_tube_stream_new (SalutConnection *conn,
       "connection", conn,
       "tubes-channel", tubes_channel,
       "object-path", object_path,
-      "xmpp-connection-manager", xmpp_connection_manager,
       "handle", handle,
       "handle-type", handle_type,
       "self-handle", self_handle,
@@ -1842,41 +1706,6 @@ salut_tube_stream_new (SalutConnection *conn,
   return obj;
 }
 
-static void
-ensure_iq_helper (SalutTubeStream *tube)
-{
-  SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (tube);
-
-  g_assert (priv->handle_type == TP_HANDLE_TYPE_CONTACT);
-
-  if (priv->iq_helper == NULL)
-    {
-      SalutXmppConnectionManagerRequestConnectionResult result;
-      SalutContactManager *contact_mgr;
-      SalutContact *contact;
-
-      g_object_get (priv->conn, "contact-manager", &contact_mgr, NULL);
-      g_assert (contact_mgr != NULL);
-
-      contact = salut_contact_manager_get_contact (contact_mgr,
-          priv->handle);
-
-      g_assert (priv->xmpp_connection_manager != NULL);
-      result = salut_xmpp_connection_manager_request_connection (
-          priv->xmpp_connection_manager, contact, &priv->xmpp_connection, NULL);
-
-      if (result ==
-          SALUT_XMPP_CONNECTION_MANAGER_REQUEST_CONNECTION_RESULT_DONE)
-        {
-          priv->iq_helper = gibber_iq_helper_new (priv->xmpp_connection);
-          g_assert (priv->iq_helper);
-        }
-
-      g_object_unref (contact);
-      g_object_unref (contact_mgr);
-    }
-}
-
 /**
  * salut_tube_stream_accept
  *
@@ -1888,7 +1717,7 @@ salut_tube_stream_accept (SalutTubeIface *tube,
 {
   SalutTubeStream *self = SALUT_TUBE_STREAM (tube);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
-  GibberXmppStanza *reply;
+  WockyStanza *reply;
 
   if (priv->state != TP_TUBE_CHANNEL_STATE_LOCAL_PENDING)
     return TRUE;
@@ -1901,17 +1730,12 @@ salut_tube_stream_accept (SalutTubeIface *tube,
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
-      ensure_iq_helper (self);
+      reply = wocky_stanza_build_iq_result (priv->iq_req, NULL);
+      wocky_porter_send (priv->conn->porter, reply);
 
-      if (priv->xmpp_connection && priv->iq_helper)
-        {
-          reply = gibber_iq_helper_new_result_reply (priv->iq_req);
-          gibber_xmpp_connection_send (priv->xmpp_connection, reply, NULL);
-
-          g_object_unref (priv->iq_req);
-          priv->iq_req = NULL;
-          g_object_unref (reply);
-        }
+      g_object_unref (priv->iq_req);
+      priv->iq_req = NULL;
+      g_object_unref (reply);
     }
 
   priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
@@ -1934,10 +1758,10 @@ salut_tube_stream_accepted (SalutTubeIface *tube)
   SalutTubeStream *self = SALUT_TUBE_STREAM (tube);
   SalutTubeStreamPrivate *priv = SALUT_TUBE_STREAM_GET_PRIVATE (self);
 
-  if (priv->state == TP_TUBE_STATE_OPEN)
+  if (priv->state == TP_TUBE_CHANNEL_STATE_OPEN)
     return;
 
-  priv->state = TP_TUBE_STATE_OPEN;
+  priv->state = TP_TUBE_CHANNEL_STATE_OPEN;
   g_signal_emit (G_OBJECT (self), signals[OPENED], 0);
 
   tp_svc_channel_interface_tube_emit_tube_channel_state_changed (
@@ -2031,12 +1855,24 @@ salut_tube_stream_listen (SalutTubeIface *tube)
 }
 
 static void
-iq_close_reply_cb (GibberIqHelper *helper,
-                   GibberXmppStanza *sent_stanza,
-                   GibberXmppStanza *reply_stanza,
-                   GObject *object,
+iq_close_reply_cb (GObject *source_object,
+                   GAsyncResult *result,
                    gpointer user_data)
 {
+  WockyPorter *porter = WOCKY_PORTER (source_object);
+  GError *error = NULL;
+  WockyStanza *stanza;
+
+  stanza = wocky_porter_send_iq_finish (porter, result, &error);
+
+  if (stanza == NULL)
+    {
+      DEBUG ("Failed to close IQ: %s", error->message);
+      g_clear_error (&error);
+      return;
+    }
+
+  g_object_unref (stanza);
 }
 
 /**
@@ -2061,42 +1897,39 @@ salut_tube_stream_close (SalutTubeIface *tube, gboolean closed_remotely)
    * contact */
   if (!closed_remotely && priv->handle_type == TP_HANDLE_TYPE_CONTACT)
     {
-      GibberXmppStanza *stanza;
-      const gchar *jid_from, *jid_to;
+      WockyStanza *stanza;
+      const gchar *jid_from;
       TpHandleRepoIface *contact_repo = tp_base_connection_get_handles (
           (TpBaseConnection *) priv->conn, TP_HANDLE_TYPE_CONTACT);
       gchar *tube_id_str;
-      GError *error = NULL;
+      SalutContactManager *contact_mgr;
+      SalutContact *contact;
 
-      jid_to = tp_handle_inspect (contact_repo, priv->handle);
       jid_from = tp_handle_inspect (contact_repo, priv->self_handle);
       tube_id_str = g_strdup_printf ("%u", priv->id);
 
-      stanza = gibber_xmpp_stanza_build (GIBBER_STANZA_TYPE_IQ,
-          GIBBER_STANZA_SUB_TYPE_SET,
-          jid_from, jid_to,
-          GIBBER_NODE, "close",
-            GIBBER_NODE_XMLNS, GIBBER_TELEPATHY_NS_TUBES,
-            GIBBER_NODE_ATTRIBUTE, "id", tube_id_str,
-          GIBBER_NODE_END,
-          GIBBER_STANZA_END);
+      g_object_get (priv->conn, "contact-manager", &contact_mgr, NULL);
+      g_assert (contact_mgr != NULL);
 
-      ensure_iq_helper (self);
+      contact = salut_contact_manager_get_contact (contact_mgr,
+          priv->handle);
 
-      if (priv->iq_helper != NULL)
-        {
-          if (!gibber_iq_helper_send_with_reply (priv->iq_helper, stanza,
-              iq_close_reply_cb, G_OBJECT(self), tube, &error))
-            {
-              DEBUG ("ERROR: Failed to send the close iq stanza: '%s'",
-                  error->message);
-              g_error_free (error);
-            }
-        }
+      stanza = wocky_stanza_build_to_contact (WOCKY_STANZA_TYPE_IQ,
+          WOCKY_STANZA_SUB_TYPE_SET,
+          jid_from, WOCKY_CONTACT (contact),
+          '(', "close",
+            ':', GIBBER_TELEPATHY_NS_TUBES,
+            '@', "id", tube_id_str,
+          ')', NULL);
+
+      wocky_porter_send_iq_async (priv->conn->porter, stanza,
+          NULL, iq_close_reply_cb, tube);
 
       g_free (tube_id_str);
 
       g_object_unref (stanza);
+      g_object_unref (contact);
+      g_object_unref (contact_mgr);
     }
 
   if (priv->handle_type == TP_HANDLE_TYPE_CONTACT)
@@ -2115,10 +1948,10 @@ salut_tube_stream_close (SalutTubeIface *tube, gboolean closed_remotely)
 }
 
 static void
-augment_si_accept_iq (GibberXmppNode *si,
+augment_si_accept_iq (WockyNode *si,
                       gpointer user_data)
 {
-  gibber_xmpp_node_add_child_ns (si, "tube", GIBBER_TELEPATHY_NS_TUBES);
+  wocky_node_add_child_ns (si, "tube", GIBBER_TELEPATHY_NS_TUBES);
 }
 
 /**
